@@ -1,18 +1,20 @@
 from .core import *
 
 from .proxy import QueueProxy
+from .data import DataFacts
 
 import logging
 
-def from_uri(queue_uri):
+def from_uri(queue_uri: str):
     """"""
     logger = logging.getLogger("from_uri")
 
+    local, remote = Queue, QueueProxy
 
     if queue_uri.startswith("http://") or queue_uri.startswith("https://"):
-        r = QueueProxy(queue_uri)
+        r = remote(queue_uri)
     else:
-        r = Queue(queue_uri)
+        r = local(queue_uri)
 
     logger.info("constructing queue from %s: %s", queue_uri, r)
     return r
