@@ -10,6 +10,8 @@ import logging
 import io
 import urllib.parse
 import traceback
+import base64
+import minio
 
 import dqueue.core 
 import dqueue.app
@@ -18,38 +20,25 @@ import dqueue.tools as tools
 import peewee # type: ignore
 import json
 
-from flask import Flask
-from flask import render_template,make_response,request,jsonify
+from flask import render_template,make_response,request,jsonify,Flask,Response
 from flasgger import Swagger, SwaggerView, Schema, fields # type: ignore
 
+import odakb
+
 decoded_entries={} # type: ignore
 
 db = dqueue.core.db
-
 app = dqueue.app.app
+auth = dqueue.app.auth
 
 
 template = {
   "swagger": "2.0",
-#  "info": {
- #   "title": "My API",
-#    "description": "API for my data",
-    #"contact": {
-    #  "responsibleOrganization": "ME",
-    #  "responsibleDeveloper": "Me",
-    #  "email": "me@me.com",
-    #  "url": "www.me.com",
-    #},
-    #"termsOfService": "http://me.com/terms",
-    #"version": "0.0.1"
- # },
-  #"host": "mysite.com",  # overrides localhost:500
   "basePath": os.environ.get("API_BASE", "/"),  # base bash for blueprint registration
   "schemes": [
     "http",
     "https"
   ],
- # "operationId": "getmyData"
 }
 
 
@@ -78,6 +67,10 @@ class TaskList(Schema):
 class LogEntry(Schema):
     message = fields.Str()
 
+class LogSummaryReport(Schema):
+    N = fields.Int()
+
+
 class QueueLogEntry(Schema):
     state = fields.Str()
     task_key = fields.Str()
@@ -91,6 +84,10 @@ class QueueList(Schema):
 class Status(Schema):
     status = fields.Str()
 
+class DataFact(Schema):
+    dag_json = fields.Str()
+    data_json = fields.Str()
+
 ## === views
 
 class TaskListView(SwaggerView):
@@ -157,12 +154,6 @@ class WorkerOffer(SwaggerView):
                     'required': False,
                     'type': 'string',
                 },
-                {
-                    'name': 'token',
-                    'in': 'query',
-                    'required': True,
-                    'type': 'string',
-                },
             ]
 
     responses = {
@@ -215,12 +206,6 @@ class WorkerAnswer(SwaggerView):
                     'required': False,
                     'type': 'string',
                 },
-                {
-                    'name': 'token',
-                    'in': 'query',
-                    'required': True,
-                    'type': 'string',
-                },
                 {
                     'name': 'task_dict',
                     'in': 'body',
@@ -250,6 +235,8 @@ class WorkerAnswer(SwaggerView):
 
         queue.task_done()
 
+        # here also upload data nad store 
+
         return jsonify(
                     { 'task_key': task.key, **task.as_dict}
                )
@@ -261,6 +248,136 @@ app.add_url_rule(
           methods=['POST']
 )
 
+## data
+
+class WorkerDataAssertFact(SwaggerView):
+    operationId = "assert_fact"
+
+    parameters = [
+                {
+                    'name': 'worker_id',
+                    'in': 'query',
+                    'required': True,
+                    'type': 'string',
+                },
+                {
+                    'name': 'payload',
+                    'in': 'body',
+                    'required': True,
+                    'schema': DataFact,
+                },
+            ]
+
+    responses = {
+            200: {
+                    'description': 'its ok',
+                 },
+            400: {
+                    'description': 'provided data insufficient',
+                 }
+            }
+
+    def post(self):
+        worker_id = dqueue.core.Queue(request.args.get('worker_id'))
+
+        payload_dict = request.json
+
+        try:
+            dag = json.loads(payload_dict['dag_json'])
+            data = json.loads(payload_dict['data_json'])
+        except (KeyError, TypeError) as e:
+            return Response(
+                        f"insufficient data: {e}",
+                        status=400,
+                    )
+
+        logger.debug("worker %s reporting fact of dag %s == %s", worker_id, len(dag), len(data))
+
+        # here also upload data and create rdf record
+
+        dag_bucket = odakb.datalake.form_bucket_name(dag)
+
+        bucket = odakb.datalake.store(
+                    dict(dag=dag, data=data),
+                    bucket_name=dag_bucket,
+                )
+
+        assert bucket == dag_bucket
+        
+
+        return jsonify(
+                { 'bucket': bucket }
+               )
+
+
+app.add_url_rule(
+         '/data/assert',
+          view_func=auth.login_required(WorkerDataAssertFact.as_view('data_assert_fact')),
+          methods=['POST']
+)
+
+class WorkerDataConsultFact(SwaggerView):
+    operationId = "consult_fact"
+
+    parameters = [
+                {
+                    'name': 'worker_id',
+                    'in': 'query',
+                    'required': True,
+                    'type': 'string',
+                },
+                {
+                    'name': 'payload',
+                    'in': 'body',
+                    'required': True,
+                    'schema': DataFact,
+                },
+            ]
+
+    responses = {
+            200: {
+                    'description': 'its ok',
+                    'schema': DataFact,
+                 },
+            400: {
+                    'description': 'no such bucket found',
+                 }
+            }
+
+    def post(self):
+        worker_id = dqueue.core.Queue(request.args.get('worker_id'))
+
+        data_dict = request.json
+
+        dag = json.loads(data_dict['dag_json'])
+
+
+        logger.info("worker %s consulting fact of dag %s", worker_id, len(dag))
+
+        dag_bucket = odakb.datalake.form_bucket_name(dag)
+        logger.info("dag bucket %s", dag_bucket)
+
+        try:
+            meta, payload  = odakb.datalake.restore(dag_bucket, return_metadata=True)
+        except minio.error.NoSuchBucket:
+            return Response(
+                      f"no such dag! bucket: {dag_bucket}",
+                      status=400,
+                   )
+
+        assert payload['dag'] == dag
+        
+        return jsonify(
+                   dag_json=json.dumps(payload['dag'], sort_keys=True),
+                   data_json=json.dumps(payload['data'], sort_keys=True),
+               )
+
+
+app.add_url_rule(
+         '/data/consult',
+          view_func=WorkerDataConsultFact.as_view('data_consult_fact'),
+          methods=['POST']
+)
 
 class TryAllLocked(SwaggerView):
     operationId = "try_all_locked"
@@ -280,12 +397,6 @@ class TryAllLocked(SwaggerView):
                     'required': False,
                     'type': 'string',
                 },
-                {
-                    'name': 'token',
-                    'in': 'query',
-                    'required': True,
-                    'type': 'string',
-                },
             ]
 
     responses = {
@@ -329,12 +440,6 @@ class ViewLogView(SwaggerView):
                     'required': False,
                     'type': 'number',
                 },
-                {
-                    'name': 'token',
-                    'in': 'query',
-                    'required': True,
-                    'type': 'string',
-                },
             ]
 
     responses = {
@@ -391,14 +496,15 @@ class ClearLog(SwaggerView):
     responses = {
             200: {
                     'description': 'task log dict',
+                    'schema': LogSummaryReport,
                 }
         }
 
     def get(self):
         queue = dqueue.core.Queue()
 
-        r = queue.clear_event_log(request.args.get('only_older_than_days', None),
-                                  request.args.get('only_kind', None))
+        r = queue.clear_event_log(request.args.get('only_older_than_days', None, type=float),
+                                  request.args.get('only_kind', None, type=str))
 
         logger.info("clear_log api clears %d entries", r)
 
@@ -446,12 +552,6 @@ class TaskLogView(SwaggerView):
                     'required': True,
                     'type': 'string',
                 },
-                {
-                    'name': 'token',
-                    'in': 'query',
-                    'required': True,
-                    'type': 'string',
-                },
             ]
 
     responses = {
@@ -514,12 +614,6 @@ class QueueLogView(SwaggerView):
                     'required': True,
                     'type': 'string',
                 },
-                {
-                    'name': 'token',
-                    'in': 'query',
-                    'required': False,
-                    'type': 'string',
-                },
             ]
 
     responses = {
@@ -566,12 +660,6 @@ class WorkerQuestion(SwaggerView):
                     'required': True,
                     'schema': TaskData,
                 },
-                {
-                    'name': 'token',
-                    'in': 'query',
-                    'required': True,
-                    'type': 'string',
-                },
                 {
                     'name': 'queue',
                     'in': 'query',
