@@ -207,33 +207,76 @@ def view(obj, follow, since=0):
 
 @logcli.command()
 @click.pass_obj
-@click.option("--before", "-b", default=None)
-@click.option("--kind", "-k", default=None)
+@click.option("--before", "-b", default=None, type=float)
+@click.option("--kind", "-k", default=None, type=str)
 def clear(obj, before, kind):
      N = obj['queue'].clear_event_log(before, kind)
      print("cleared", N)
 
+###
 
+@cli.group("data")
+def datacli():
+    pass
+
+@datacli.command("assert")
+@click.pass_obj
+@click.option("-g", "--dag", type=click.File('rt'))
+@click.option("-d", "--data", type=click.File('rt'))
+def assert_fact(obj, dag, data):
+    dag = json.load(dag)
+    data = json.load(data)
+
+    obj['queue'].assert_fact(
+                dag=dag,
+                data=data,
+            )
+
+@datacli.command()
+@click.pass_obj
+@click.option("-g", "--dag", type=click.File('rt'))
+@click.option("-o", "--output", type=click.File('wt'))
+def consult(obj, dag, output):
+    dag = json.load(dag)
+
+    d = obj['queue'].consult_fact(dag=dag)
+
+    json.dump(d, output)
+
+    
+
+@datacli.command()
+@click.pass_obj
+def list_facts(obj):
+    pass
+
+###
 
 @cli.command()
 @click.option('-w', '--watch', default=None, type=int)
 @click.pass_obj
-def try_all_locked(obj, watch):
+def guardian(obj, watch):
     while True:
+        #try_all_locked
         print("trying to unlock something")
         task_data=obj['queue'].try_all_locked()
         print(colored("unlocked:", "green"), task_data)
-    
-        if not watch:
-            break
-
         
+        #clear event log 
+        N = obj['queue'].clear_event_log(only_older_than_days=2./24.)
+        print(f"cleared event log of {N} entries")
+
+        # stats
+
         print("getting queue statistics...")
         for k,v in obj['queue'].info.items():
             print(k, ":", len(v), end="; ")
         print("\n")
 
         print("sleeping", watch)
+
+        if not watch:
+            break
         time.sleep(watch)
 
 @cli.command()
@@ -259,6 +302,7 @@ def resubmit(obj, scope_selector):
     r = obj['queue'].resubmit(scope, selector).response().result
     print(colored("resubmitted:", "green"), ":", r)
 
+
 def main():
     cli(obj={})
 
